package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.itextpdf.text.DocumentException;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.entity.BasicAttachment;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * <p>
 * 所有附件记录表 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface BasicAttachmentService extends IService<BasicAttachment> {

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @return
     */
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type);

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    public BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type, Long objId);

    /**
     * 保存html附件
     *
     * @param fileName
     * @param htmlContent
     * @param userId
     * @param localFileList
     * @return
     * @throws IOException
     */
    public BasicAttachment saveAttachmentHtml(String fileName, String htmlContent, Long userId, List<PdfDto> localFileList) throws IOException;

    /**
     * 保存附件
     *
     * @param dirName
     * @param userId
     * @return
     * @throws IOException
     */
    public BasicAttachment saveAttachmentPdf(String dirName, Long userId) throws IOException;

    /**
     * 保存附件
     *
     * @param basicAttachment
     * @param pdfList
     * @param printCount
     * @return
     * @throws IOException
     */
    public BasicAttachment saveAttachmentPdf(BasicAttachment basicAttachment, List<PdfDto> pdfList, Integer printCount) throws IOException, DocumentException;

    /**
     * 删除附件
     *
     * @param tbAttachment
     */
    public void deleteAttachment(BasicAttachment tbAttachment);

    /**
     * 批量删除附件
     *
     * @param basicAttachmentList
     */
    public void batchDeleteAttachment(List<BasicAttachment> basicAttachmentList);
}
